@@ -1,23 +1,27 @@
 # coding: utf-8
 
-import abc
+import atexit
 import base64
-import enum
-import functools
 import io
 import json
+import subprocess
+import sys
+import threading
+import time
 import typing
 
-from logzero import setup_logger
 import requests
 from cached_property import cached_property
+from logzero import setup_logger
 from PIL import Image
 
+from ._alert import Alert
+from ._base import BaseClient
+from ._logger import logger
 from ._proto import *
 from ._types import *
 from .exceptions import *
-
-logger = setup_logger("wdapy")
+from .usbmux import requests_usbmux, usbmux
 
 
 class HTTPResponse:
@@ -29,15 +33,13 @@ class HTTPResponse:
         return self._err is None and self._resp.status_code == 200
 
     def json(self) -> dict:
+        assert self._resp is not None
         try:
             return self._resp.json()
         except json.JSONDecodeError:
             return RequestError("JSON decode error", self._resp.text)
 
     def get_error_message(self) -> str:
-        # if self._err:
-        #     return str(self._err)
-        # self.json()
         if self._resp:
             return self._resp.text
         return str(self._err)
@@ -49,14 +51,22 @@ class HTTPResponse:
             raise RequestError(self._resp.status_code, self._resp.text)
 
 
-class CommonClient(abc.ABC):
+class CommonClient(BaseClient):
     def __init__(self, wda_url: str):
-        self._wda_url = wda_url
-        self._request_timeout = None
-
-        self._session_id: str = None
-        self._recover: Recover = None
+        super().__init__(wda_url)
         self.__ui_size = None
+        self.__debug = False
+
+    @property
+    def debug(self) -> bool:
+        return self.__debug
+
+    @debug.setter
+    def debug(self, v: bool):
+        if v:
+            setup_logger(NAME)
+        else:
+            setup_logger(NAME, level=logging.INFO)
 
     def app_start(self, bundle_id: str):
         self.session_request(POST, "/wda/apps/launch", {
@@ -83,15 +93,35 @@ class CommonClient(abc.ABC):
         value = data['value']
         return AppInfo.value_of(value)
 
-    def status(self) -> StatusInfo:
-        data = self.request(GET, "/status")
-        return StatusInfo.value_of(data)
+    def app_list(self) -> AppList:
+        value = self.session_request(GET, "/wda/apps/list")["value"][0]
+        return AppList.value_of(value)
+
+    def deactive(self, duration: float):
+        self.session_request(POST, "/wda/deactivateApp", {
+            "duration": duration
+        })
+
+
+    @cached_property
+    def alert(self) -> Alert:
+        return Alert(self)
+
+    def sourcetree(self) -> SourceTree:
+        data = self.request(GET, "/source")
+        return SourceTree.value_of(data)
 
     def open_url(self, url: str):
         self.session_request(POST, "/url", {
             "url": url
         })
 
+    def set_clipboard(self, content: str, content_type="plaintext"):
+        self.session_request(POST, "/wda/setPasteboard",{
+            "content": base64.b64encode(content.encode()).decode(),
+            "contentType": content_type
+        })
+
     def appium_settings(self, kwargs: dict = None) -> dict:
         if kwargs is None:
             return self.session_request(GET, "/appium/settings")["value"]
@@ -110,6 +140,9 @@ class CommonClient(abc.ABC):
     def homescreen(self):
         self.request(POST, "/wda/homescreen")
 
+    def shutdown(self):
+        self.request(GET, "/wda/shutdown")
+
     def get_orientation(self) -> Orientation:
         value = self.session_request(GET, '/orientation')['value']
         return Orientation(value)
@@ -138,6 +171,7 @@ class CommonClient(abc.ABC):
     def tap(self, x: int, y: int):
         self.session_request(POST, "/wda/tap/0", {"x": x, "y": y})
 
+
     def swipe(self,
               from_x: int,
               from_y: int,
@@ -152,6 +186,33 @@ class CommonClient(abc.ABC):
             "duration": duration}
         self.session_request(POST, "/wda/dragfromtoforduration", payload)
 
+
+    def press(self, name: Keycode):
+        payload = {
+            "name": name
+        }
+        self.session_request(POST, "/wda/pressButton", payload)
+
+    def press_duration(self, name: Keycode, duration: float):
+        hid_usages = {
+            "home": 0x40,
+            "volumeup": 0xE9,
+            "volumedown": 0xEA,
+            "power": 0x30,
+            "snapshot": 0x65,
+            "power_plus_home": 0x65
+        }
+        name = name.lower()
+        if name not in hid_usages:
+            raise ValueError("Invalid name:", name)
+        hid_usages = hid_usages[name]
+        payload = {
+            "page": 0x0C,
+            "usage": hid_usages,
+            "duration": duration
+        }
+        return self.session_request(POST, "/wda/performIoHidEvent", payload)
+
     @cached_property
     def scale(self) -> int:
         # Response example
@@ -159,6 +220,12 @@ class CommonClient(abc.ABC):
         value = self.session_request(GET, "/wda/screen")['value']
         return value['scale']
 
+    def status_barsize(self) -> StatusBarSize:
+        # Response example
+        # {"statusBarSize": {'width': 320, 'height': 20}, 'scale': 2}
+        value = self.session_request(GET, "/wda/screen")['value']
+        return StatusBarSize.value_of(value['statusBarSize'])
+
     def screenshot(self) -> Image.Image:
         """ take screenshot """
         value = self.request(GET, "/screenshot")["value"]
@@ -167,81 +234,59 @@ class CommonClient(abc.ABC):
         im = Image.open(buf)
         return im.convert("RGB")
 
-    def session(self,
-                bundle_id: str = None,
-                arguments: list = None,
-                environment: dict = None) -> str:
-        """ create session and return session id """
-        capabilities = {}
-        if bundle_id:
-            always_match = {
-                "bundleId": bundle_id,
-                "arguments": arguments or [],
-                "environment": environment or {},
-                "shouldWaitForQuiescence": False,
-            }
-            capabilities['alwaysMatch'] = always_match
-        payload = {
-            "capabilities": capabilities,
-            "desiredCapabilities": capabilities.get('alwaysMatch',
-                                                    {}),  # 兼容旧版的wda
-        }
-        data = self.request(POST, "/session", payload)
+    def battery_info(self) -> BatteryInfo:
+        data = self.session_request(GET, "/wda/batteryInfo")["value"]
+        return BatteryInfo.value_of(data)
 
-        # update cached session_id
-        self._session_id = data['sessionId']
-        return self._session_id
+    @property
+    def info(self) -> DeviceInfo:
+        return self.device_info()
 
-    def set_recover_handler(self, recover: Recover):
-        self._recover = Recover
+    def device_info(self) -> DeviceInfo:
+        data = self.session_request(GET, "/wda/device/info")["value"]
+        return DeviceInfo.value_of(data)
 
-    def _get_valid_session_id(self) -> str:
-        if self._session_id:
-            return self._session_id
-        old_session_id = self.status().session_id
-        if old_session_id:
-            self._session_id = old_session_id
-        else:
-            self._session_id = self.session()
-        return self._session_id
 
-    def session_request(self, method: RequestMethod, urlpath: str, payload: dict = None) -> dict:
-        session_id = self._get_valid_session_id()
-        urlpath = f"/session/{session_id}/" + urlpath.lstrip("/")
-        return self.request(method, urlpath, payload)
 
-    def request(self, method: RequestMethod, urlpath: str, payload: dict = None) -> dict:
-        full_url = self._wda_url.rstrip("/") + "/" + urlpath.lstrip("/")
-        logger.debug("$ %s", f"curl -X{method} {full_url} -d {payload!r}")
-        resp = self._request_with_error(method, full_url, payload)
-        if not resp.is_success():
-            if self._recover and not self._recover.recover():
-                raise RequestError(
-                    "recover failed", resp.get_error_message())
-            resp = self._request_with_error(method, full_url, payload)
+class XCUITestRecover(Recover):
+    def __init__(self, udid: str):
+        self._udid = udid
 
-        short_json = resp.json().copy()
-        for k, v in short_json.items():
-            if isinstance(v, str) and len(v) > 40:
-                v = v[:20] + "... skip ..." + v[-10:]
-            short_json[k] = v
-        logger.debug("==> Response <==\n%s", json.dumps(short_json, indent=4))
-
-        value = resp.json().get("value")
-        if value and isinstance(value, dict) and value.get("error"):
-            raise ApiError(value["error"], value.get("message"))
-
-        return resp.json()
-
-    def _request_with_error(self, method: RequestMethod, url: str, payload: dict, **kwargs) -> HTTPResponse:
-        _request = functools.partial(
-            requests.request, method, url, json=payload)
-        try:
-            resp = _request(**kwargs)
-            return HTTPResponse(resp, None)
-        except requests.RequestException as err:
-            return HTTPResponse(err.response, err)
+    def recover(self) -> bool:
+        """ launch by tidevice
+        
+        https://github.com/alibaba/tidevice
+        """
+        logger.info("WDA is starting using tidevice ...")
+        args = [sys.executable, '-m', 'tidevice', '-u', self._udid, 'xctest']
+        p = subprocess.Popen(args,
+                         stdin=subprocess.DEVNULL,
+                         stdout=subprocess.PIPE,
+                         stderr=subprocess.STDOUT,
+                         start_new_session=True,
+                         close_fds=True, encoding="utf-8")
+        
+        success = False
+        deadline = time.time() + 10
+        while time.time() < deadline:
+            if p.poll() is not None:
+                logger.warning("xctest exit, output: %s", p.stdout.read())
+                return False
+            line = p.stdout.readline().strip()
+            # logger.info("%s", line)
+            if "WebDriverAgent start successfully" in line:
+                logger.info("WDA started")
+                success = True
+                break
 
+        def drain_stdout():
+            while p.stdout.read() != "":
+                pass
+        
+        if success:
+            threading.Thread(target=drain_stdout, daemon=True).start()
+            atexit.register(p.kill)
+        return success
 
 class AppiumClient(CommonClient):
     """
@@ -252,7 +297,16 @@ class AppiumClient(CommonClient):
         super().__init__(wda_url)
 
 
-class NanoscopicClient(AppiumClient):
+class AppiumUSBClient(AppiumClient):
+    def __init__(self, udid: str = None, port: int = 8100):
+        if udid is None:
+            _usbmux = usbmux.Usbmux()
+            udid = _usbmux.get_single_device_udid()
+        super().__init__(requests_usbmux.DEFAULT_SCHEME+udid+f":{port}")
+        self.set_recover_handler(XCUITestRecover(udid))
+
+
+class NanoClient(AppiumClient):
     """
     Repo: https://github.com/nanoscopic/WebDriverAgent
 
@@ -281,3 +335,12 @@ class NanoscopicClient(AppiumClient):
             "x2": to_x,
             "y2": to_y,
             "delay": duration})
+
+
+class NanoUSBClient(NanoClient):
+    def __init__(self, udid: str = None, port: int = 8100):
+        if udid is None:
+            _usbmux = usbmux.Usbmux()
+            udid = _usbmux.get_single_device_udid()
+        super().__init__(requests_usbmux.DEFAULT_SCHEME+udid+f":{port}")
+        self.set_recover_handler(XCUITestRecover(udid))
